#include <stdlib.h>
#include <string.h>
#include "../include/generator.h"
#include "../include/utils.h"

static void place_centers(GameBoard *board);

static int is_adjacent_to_center(const GameBoard *board, int x, int y) {
    for (int dx = -1; dx <= 1; dx++) {
        for (int dy = -1; dy <= 1; dy++) {
            if (dx == 0 && dy == 0) continue;
            int nx = x + dx, ny = y + dy;
            if (is_valid_coord(board, nx, ny) && get_cell(board, nx, ny) == CELL_CENTER) {
                return 1;
            }
        }
    }
    return 0;
}

static void grow_galaxy(GameBoard *board, int x, int y, int galaxy_id) {
    Point center = {x, y};
    int directions[4][2] = {{0, 1}, {1, 0}, {0, -1}, {-1, 0}};
    
    for (int i = 0; i < 4; i++) {
        int dx = directions[i][0], dy = directions[i][1];
        int nx = x + dx, ny = y + dy;
        Point sym = get_symmetric_point(board, center, (Point){nx, ny});
        int sym_x = sym.x, sym_y = sym.y;
        
        if (is_valid_coord(board, nx, ny) && is_valid_coord(board, sym_x, sym_y) &&
            get_cell(board, nx, ny) == CELL_EMPTY && get_cell(board, sym_x, sym_y) == CELL_EMPTY) {
            set_cell(board, nx, ny, galaxy_id);
            set_cell(board, sym_x, sym_y, galaxy_id);
            grow_galaxy(board, nx, ny, galaxy_id);
        }
    }
}

static void generate_galaxies(GameBoard *board) {
    int galaxy_id = 2;
    for (int y = 0; y < board->height; y++) {
        for (int x = 0; x < board->width; x++) {
            if (get_cell(board, x, y) == CELL_CENTER) {
                grow_galaxy(board, x, y, galaxy_id++);
            }
        }
    }
}

static void generate_symmetry_lines(GameBoard *board) {
    for (int y = 0; y < board->height; y++) {
        for (int x = 0; x < board->width; x++) {
            if (get_cell(board, x, y) == CELL_CENTER) {
                for (int i = 0; i < 4; i++) {
                    int nx = x, ny = y;
                    while (1) {
                        nx += (i == 1) - (i == 3);
                        ny += (i == 0) - (i == 2);
                        if (!is_valid_coord(board, nx, ny) || get_cell(board, nx, ny) != get_cell(board, x, y)) break;
                        set_cell(board, nx, ny, get_cell(board, nx, ny) | CELL_SYMMETRY);
                    }
                }
            }
        }
    }
}

static int find_neighbor_galaxy(const GameBoard *board, int x, int y) {
    int directions[4][2] = {{0, 1}, {1, 0}, {0, -1}, {-1, 0}};
    for (int i = 0; i < 4; i++) {
        int nx = x + directions[i][0], ny = y + directions[i][1];
        if (is_valid_coord(board, nx, ny) && get_cell(board, nx, ny) != CELL_EMPTY) {
            return get_cell(board, nx, ny) & ~CELL_SYMMETRY;
        }
    }
    return CELL_EMPTY;
}

int generate_puzzle(GameBoard *board) {
    memset(board->cells, CELL_EMPTY, board->width * board->height);
    
    place_centers(board);
    generate_galaxies(board);
    generate_symmetry_lines(board);
    
    for (int y = 0; y < board->height; y++) {
        for (int x = 0; x < board->width; x++) {
            if (get_cell(board, x, y) == CELL_EMPTY) {
                int neighbor = find_neighbor_galaxy(board, x, y);
                if (neighbor == CELL_EMPTY) return 0;
                set_cell(board, x, y, neighbor);
            }
        }
    }
    
    return 1;
}

GameBoard *create_board(uint8_t width, uint8_t height) {
    GameBoard *board = malloc(sizeof(GameBoard));
    if (!board) return NULL;

    board->width = width;
    board->height = height;
    board->cells = calloc(width * height, sizeof(uint8_t));
    board->centers = malloc(sizeof(Point) * (width * height / 25));
    board->center_count = 0;

    if (!board->cells || !board->centers) {
        free(board->cells);
        free(board->centers);
        free(board);
        return NULL;
    }

    return board;
}

void destroy_board(GameBoard *board) {
    if (board) {
        free(board->cells);
        free(board->centers);
        free(board);
    }
}

static void place_centers(GameBoard *board) {
    int centers = board->width * board->height / 25;
    while (centers > 0) {
        int x = random_int(0, board->width - 1);
        int y = random_int(0, board->height - 1);
        if (get_cell(board, x, y) == CELL_EMPTY && !is_adjacent_to_center(board, x, y)) {
            set_cell(board, x, y, CELL_CENTER);
            board->centers[board->center_count++] = (Point){x, y};
            centers--;
        }
    }
}